<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\FIN\Modules\ContaReceber\Factory\GeralRecTipoajreceberFactory;

class GeralRecTipoajreceberService{

    protected $environment;
    protected $newCode;
    protected $entityManager;
    protected $factory;

    public function __construct($environment, $newCode, EntityManager $entityManager,
                                GeralRecTipoajreceberFactory $factory) {
        $this->environment = $environment;
        $this->newCode = $newCode;
        $this->entityManager = $entityManager;
        $this->factory = $factory;
	}

    public function insereParametroContaReceber($idbloqincntr, $idtpcntrlib, $idutctrlcanremr,
                                     $idbloqaltbolrec, $idutlnfdevolrec, $idutbcoprefclie,
                                     $idextelacalajubx){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $cdgeralrec = $this->newCode->getCodeByTableName('GERAL_REC',$this->environment->getNrOrgTrab());
            $geralrec = $this->factory->insereParametroContaReceber(intVal($cdgeralrec), $idbloqincntr, $idtpcntrlib, $idutctrlcanremr,
                $idbloqaltbolrec, $idutlnfdevolrec, $idutbcoprefclie, $idextelacalajubx,$this->environment->getNrOrgTrab());
            $this->entityManager->persist($geralrec);
            $this->entityManager->flush();

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function saveTabedsOperadores($row){
        try {
            
            $paramsGeralRec = array(
                   
               "IDBLOQINCNTR" => $row[0]["IDBLOQINCNTR"],
               "CDGROPERCNTR" => $row[0]["CDGROPERCNTR"],
               "IDTPCNTRLIB" => $row[0]["IDTPCNTRLIB"],
               "IDUTLPRORVCREC" => $row[0]["IDUTLPRORVCREC"],
               "IDUTCTRLCANREMR" => $row[0]["IDUTCTRLCANREMR"],
               "CDGERALREC" => 1
            );
            
            $this->factory->atualizaTabelaGeralRec($paramsGeralRec);
            
            $paramsParamGeral = array(
               "CDGROPERPROR" => $row[0]["CDGROPERPROR"],
               "IDUTLPROCREC" => $row[0]["IDUTLPROCREC"],
               "CDGROPERRECCONTR" => $row[0]["CDGROPERRECCONTR"],
               "IDCONTROLARECCONTRATO" => $row[0]["IDCONTROLARECCONTRATO"],
               "CDPARGERAL" => "0001"
            );

            $this->factory->atualizaTabelaParamGeral($paramsParamGeral);
            
            if(empty($row[0]["CDGRUPOPERCONT"])) {
                $idautgrupop = 'PRVE';
                $verificaOperadoresInseridos = $this->factory->verificaOperadoresInseridos($idautgrupop);
                
                foreach($verificaOperadoresInseridos as $dados){
                    $param =  array(
                        "CDGRUPOPER" => $dados["CDGRUPOPER"]
                    );
                    $this->factory->removeGrupoOperadores($param);
                }
            } else {
                
                // Adiciona ou remove (Grupo de Operadores) referente a Controla Prorrogacao de Vencimento
                $idautgrupop = 'PRVE';
                $verificaOperadoresInseridos = $this->factory->verificaOperadoresInseridos($idautgrupop);
                $operadoresExistentes = array();
                
                foreach($verificaOperadoresInseridos as $dados){
                    array_push($operadoresExistentes, $dados["CDGRUPOPER"]);
                    if(!in_array($dados["CDGRUPOPER"],$row[0]["CDGRUPOPERCONT"])){
                        $param =  array(
                            "CDGRUPOPER" => $dados["CDGRUPOPER"]
                        );
                        $this->factory->removeGrupoOperadores($param);
                    }
                }
    
                $nrautgrupop = $this->newCode->getCodeByTableName('AUTGRUPOP',$this->environment->getNrOrgTrab());
                foreach ($row[0]["CDGRUPOPERCONT"] as $indiceContPross) {
                    if(!in_array($indiceContPross,$operadoresExistentes)){
                        if(empty($verificaOperadoresInseridos)){
                            $dados = array(
                              "CDGRUPOPER" => $indiceContPross,
                              "IDAUTGRUPOP" => "PRVE",
                              "NRAUTGRUPOP" => substr($nrautgrupop, -3)
                            );
                            $this->factory->insereAutoGroupPop($dados);
                            
                        } else {
                            $dados = array(
                              "CDGRUPOPER" => $indiceContPross,
                              "IDAUTGRUPOP" => "PRVE",
                              "NRAUTGRUPOP" => $verificaOperadoresInseridos[0]["NRAUTGRUPOP"]
                            );
                            $this->factory->insereAutoGroupPop($dados);
                        }
                        
                    }
                }
            }
            
            if(empty($row[0]["CDGRUPOPEREXCLUSAOBAIXA"])){
                
                $idautgrupop = 'EXBR';
                $verificaOperadoresExclusaoBaixa = $this->factory->verificaOperadoresInseridos($idautgrupop);
    
                foreach($verificaOperadoresExclusaoBaixa as $dados){
                    $param =  array(
                        "CDGRUPOPER" => $dados["CDGRUPOPER"]
                    );
                    $this->factory->removeGrupoOperadores($param);
                }
                
            } else {
                // Adiciona ou remove (Grupo de Operadores) referente a Controla Exclusao de Baixa (Recebimento)
                $idautgrupop = 'EXBR';
                $verificaOperadoresExclusaoBaixa = $this->factory->verificaOperadoresInseridos($idautgrupop);
                $operadoresExclusaoBaixa = array();
    
                foreach($verificaOperadoresExclusaoBaixa as $dados){
                    array_push($operadoresExclusaoBaixa, $dados["CDGRUPOPER"]);
                    if(!in_array($dados["CDGRUPOPER"],$row[0]["CDGRUPOPEREXCLUSAOBAIXA"])){
                        $param =  array(
                            "CDGRUPOPER" => $dados["CDGRUPOPER"]
                        );
                        $this->factory->removeGrupoOperadores($param);
                    }
                }
                
                $nrautgrupop = $this->newCode->getCodeByTableName('AUTGRUPOP',$this->environment->getNrOrgTrab());
                foreach ($row[0]["CDGRUPOPEREXCLUSAOBAIXA"] as $indiceExcBaixa) {
                    if(!in_array($indiceExcBaixa,$operadoresExclusaoBaixa)){
                        if(empty($verificaOperadoresExclusaoBaixa)){
                            $dados = array(
                              "CDGRUPOPER" => $indiceExcBaixa,
                              "IDAUTGRUPOP" => "EXBR",
                              "NRAUTGRUPOP" =>  substr($nrautgrupop, -3)
                            );
                            $this->factory->insereAutoGroupPop($dados);
                            
                        } else {
                            $dados = array(
                              "CDGRUPOPER" => $indiceExcBaixa,
                              "IDAUTGRUPOP" => "EXBR",
                              "NRAUTGRUPOP" => $verificaOperadoresExclusaoBaixa[0]["NRAUTGRUPOP"]
                            );
                            $this->factory->insereAutoGroupPop($dados);
                        }
                        
                    }
                }
            }
            
            if(empty($row[0]["CDGRUPOPERCANREMBOL"])){
                
                $idautgrupop = 'CRMR';
                $verificaOperadoresRemessaBoleto = $this->factory->verificaOperadoresInseridos($idautgrupop);
                
                foreach($verificaOperadoresRemessaBoleto as $dados){
                    $param =  array(
                        "CDGRUPOPER" => $dados["CDGRUPOPER"]
                    );
                    $this->factory->removeGrupoOperadores($param);
                }
                
            } else {
                
                // Adiciona ou remove (Grupo de Operadores) referente a Controla Cancelamento de Remessa de Boleto
                $idautgrupop = 'CRMR';
                $verificaOperadoresRemessaBoleto = $this->factory->verificaOperadoresInseridos($idautgrupop);
                $operadoresRemessaBoleto = array();
                
                foreach($verificaOperadoresRemessaBoleto as $dados){
                    array_push($operadoresRemessaBoleto, $dados["CDGRUPOPER"]);
                    if(!in_array($dados["CDGRUPOPER"],$row[0]["CDGRUPOPERCANREMBOL"])){
                        $param =  array(
                            "CDGRUPOPER" => $dados["CDGRUPOPER"]
                        );
                        $this->factory->removeGrupoOperadores($param);
                    }
                }
                
                $nrautgrupop = $this->newCode->getCodeByTableName('AUTGRUPOP',$this->environment->getNrOrgTrab());
                foreach ($row[0]["CDGRUPOPERCANREMBOL"] as $indiceRemBoleto) {
                    if(!in_array($indiceRemBoleto,$operadoresRemessaBoleto)){
                        if(empty($verificaOperadoresRemessaBoleto)){
                            $dados = array(
                              "CDGRUPOPER" => $indiceRemBoleto,
                              "IDAUTGRUPOP" => "CRMR",
                              "NRAUTGRUPOP" => substr($nrautgrupop, -3)
                            );
                            $this->factory->insereAutoGroupPop($dados);
                        } else{
                            $dados = array(
                              "CDGRUPOPER" => $indiceRemBoleto,
                              "IDAUTGRUPOP" => "CRMR",
                              "NRAUTGRUPOP" => $verificaOperadoresRemessaBoleto[0]["NRAUTGRUPOP"]
                            );
                            $this->factory->insereAutoGroupPop($dados);
                        }
                    }
                }
            }
            
        } catch (\Exception $e) {
            throw $e;
        }
    }
}