<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Controllers;

use Teknisa\FIN\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;

class Bco54200ExtratoBancario extends TeknisaCrud {

    protected $dataSourceName = 'movreal_banco_contcorr';
    protected $dataSourceNameToSave = 'movreal';
    protected $parameterBagColumns = array();
    protected $Bco54200Service;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $Bco54200Service, $entityManager, $masks){
    	parent::__construct($crudService, $parameterBag);
    	$this->Bco54200Service = $Bco54200Service;
    	$this->entityManager = $entityManager;
    	$this->masks = $masks;
    }


    /****************************** Extrato Bancário ***********************************/

    /**
     * Essa função tem como finalidade salvar as movimentações que forem inseridas ou
     * editadas na tela. É necessário fazer algumas validações antes de salvar o registro
     * na tabela.
     *
     * @param  DTO\Request  $request  row, ou conjunto do rows (dataset) que sera passado do front
     * @param  DTO\Response $response o que sera retornado para o backend
     */
    public function save(DTO\Request $request, DTO\Response $response) {
    	//Pegando as rows para serem salvas
    	$dataSet = $request->getDataSet();
    	$rows = $dataSet->getRows();
    	
    	//Criando uma transaction
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $movimentosDuplicados = array();
        try {
            // removido esse trecho de código, não fez diferença remover ou deixar
            // if($rows[0]['__is_new']){
            //     $rows = $this->Bco54200Service->removeMovimentosExistentesNoSistema($rows);
                
            //     $rowsArray = $this->Bco54200Service->removeMovimentosDuplicadosNoArquivo($rows);
                
            //     $rows = $rowsArray[0];
               
            //     $movimentosDuplicados = $rowsArray[1];
            // }
            if (count($rows)){
                 
                foreach($rows as $row) {
                    //Verifica se o operador logado pode fazer essa operação
                    $this->Bco54200Service->validaOperador($row['CDREDUCONT']);

                    //Faz as validações referentes a tabela de extrato, EXTRBANCO
                    $row = $this->Bco54200Service->validaExtrato($row);
                    $this->entityManager->flush();
                    
                    $extratoImportado = $this->Bco54200Service->getExtrato($row);
                    
                    //Quando novo, insere a movimentação na tabela MOVREAL, quando edição, apenas
                    //updata o registro
                    if($row['__is_new']){
                        if(empty($extratoImportado)){
                            $this->Bco54200Service->insertMovReal($row);
                        } else{
                            array_push($movimentosDuplicados,$extratoImportado[0]);
                        }
                    } else {
                        $this->Bco54200Service->updateMovReal($row);
                    }
                    $this->entityManager->flush();

                    //Updata o extrato, recalculando o seu valor final.
                    $this->Bco54200Service->recalculoExtrato($row);
                    $this->entityManager->flush();

        		    //A geração de log foi cancelada ou existe deletecascae que impede o log
                    //$this->Bco54200Service->geraLog($row);
                    //$this->entityManager->flush();
                }
            }else{
                $response->setError(new Error("Os dados do arquivo ja foram importados no sistema.", 500));
            }
            $connection->commit();
            $response->addDataSet(new DataSet("MOVDUPLICADOS", $movimentosDuplicados));
    	} catch (\Exception $e) {
            $connection->rollback();
    		$response->setError(new Error($e->getMessage(), $e->getCode()));
    	}
    }

    /**
     * Essa função tem como finalidade exclui a movimentação. Depois ela chama uma outra
     * função para fazer as validações do extrato bancario.
     *
     * @param  DTO\Request  $request  row, ou conjunto do rows (dataset) que sera passado do front
     * @param  DTO\Response $response o que sera retornado para o backend
     */
    public function delete(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $this->Bco54200Service->deleteMovreal($row);
            $this->entityManager->flush();
            $this->Bco54200Service->recalculoExtrato($row);
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Essa função tem como finalidade buscar os registros usando o find do parent, e
     * formatar as rows aplicando as mascaras de agencia e conta de cada registro. So é
     * aplicada a mascara quando o numero de caracteres do resgistro e da mascara são iguais,
     * o que significa que o resgistro foi inserido junto aquele modelo de mascara.
     *
     * @param  DTO\Request\Filter  $request  row, ou conjunto do rows (dataset) que sera passado do front
     * @param  DTO\Response        $response o que sera retornado para o backend
     */
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        parent::find($request, $response);
        $dataSets = $response->getDataSets();

        if(count($dataSets) > 0) {
            $rows = $dataSets[0]->getRows();
            for($i = 0; count($rows) > $i; $i++){

                //Retira pontos (.) e traços (-) da mascará para comparação de caracteres
                $matches = array();
                preg_match_all('/[^-.]*/',$rows[$i]['CDPICTAGEN'], $matches);
                $unmaskedString = implode('', $matches[0]);

                if(strlen($rows[$i]['CDAGENCIA']) === strlen($unmaskedString)) {
                    $rows[$i]['CDAGENCIAMASK'] = $this->masks->mask($rows[$i]['CDAGENCIA'], $rows[$i]['CDPICTAGEN']);
                } else {
                    $rows[$i]['CDAGENCIAMASK'] = $rows[$i]['CDAGENCIA'];
                }

                //Retira pontos (.) e traços (-) da mascará para comparação de caracteres
                $matches = array();
                preg_match_all('/[^-.]*/',$rows[$i]['CDPICTCONT'], $matches);
                $unmaskedString = implode('', $matches[0]);

                if (strlen($rows[$i]['CDCONTCORR']) === strlen($unmaskedString)) {
                    $rows[$i]['CDCONTCORRMASK'] = $this->masks->mask($rows[$i]['CDCONTCORR'], $rows[$i]['CDPICTCONT']);
                } else {
                    $rows[$i]['CDCONTCORRMASK'] = $rows[$i]['CDCONTCORR'];
                }
            }

            $newDataSet = new DataSet($this->dataSourceName, $rows);
            $response->setDataSets(array($newDataSet));
        }
    }

    /**************************** Importação de Extrato Bancário ************************/

    /**
     * Essa função tem como finalidade verificar se o arquivo importado é do tipo TXT ou
     * OFX e chamar a função comrrespindende para lelos. Ambas as funções iram retornar
     * um array de duas posições, onde a primeira se trata dos dados do cabeçalho da
     * widget de imporatação, e a segunda posição as movimentações que deverão ser
     * salvas, e que iram aparecer no grid.
     *
     * @param  DTO\Request\Row  $request  row, ou conjunto do rows (dataset) que sera passado do front
     * @param  DTO\Response     $response o que sera retornado para o backend
     */
    public function readFile(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            if($row['TIPO'] === 'TXT'){
                $dataSets = $this->Bco54200Service->readFileTxt($row['CONTEUDO'], $row['BANCO']);
            } else {
                $dataSets = $this->Bco54200Service->readFileOfx($row['CONTEUDO'], $row['EMPRESA']);
            }
            $dataSetExtr = new DataSet('extrbanco', $dataSets[0]);
            $dataSetMov = new DataSet('movreal', $dataSets[1]);
            $response->setDataSets(array($dataSetExtr, $dataSetMov));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
       public function saveDuplicado(DTO\Request $request, DTO\Response $response) {
        
    	//Pegando as rows para serem salvas
    	$dataSet = $request->getDataSet();
    	$rows = $dataSet->getRows();
        
    	//Criando uma transaction
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $movimentosDuplicados = array();

        try {
            if (count($rows)){
                foreach($rows as $row) {
                    //Verifica se o operador logado pode fazer essa operação
                    $this->Bco54200Service->validaOperador($row['CDREDUCONT']);

                    //Faz as validações referentes a tabela de extrato, EXTRBANCO
                    $row = $this->Bco54200Service->validaExtrato($row);
                    $this->entityManager->flush();

                    
                    //updata o registro
                    $this->Bco54200Service->insertMovReal($row);
                    $this->entityManager->flush();

                    //Updata o extrato, recalculando o seu valor final.
                    $this->Bco54200Service->recalculoExtrato($row);
                    $this->entityManager->flush();

        		   
                }
            }
            $connection->commit();
            $response->addDataSet(new DataSet("success", []));
    	} catch (\Exception $e) {
            $connection->rollback();
    		$response->setError(new Error($e->getMessage(), $e->getCode()));
    	}
    }
    
    public function buscaExtratoImportado(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $buscaConciliacao = $this->Bco54200Service->buscaExtratoImportado($row);
        $response->addDataSet(new DataSet('conciliacao', ['conciliacao' => $buscaConciliacao]));
    }
}
